/*-----------------------------------------------------------------------*/
/* Low level disk I/O module skeleton for FatFs     (C)ChaN, 2019        */
/*-----------------------------------------------------------------------*/
/* If a working storage control module is available, it should be        */
/* attached to the FatFs via a glue function rather than modifying it.   */
/* This is an example of glue functions to attach various exsisting      */
/* storage control modules to the FatFs module with a defined API.       */
/*-----------------------------------------------------------------------*/

#include "ff.h"			/* Obtains integer types */
#include "diskio.h"		/* Declarations of disk functions */

#include "flash.h"
#include "rtc.h"

/* Definitions of physical drive number for each drive */
//#define DEV_RAM		0	/* Example: Map Ramdisk to physical drive 0 */
//#define DEV_MMC		1	/* Example: Map MMC/SD card to physical drive 1 */
//#define DEV_USB		2	/* Example: Map USB MSD to physical drive 2 */
#define DEV_FLASH   0

/*-----------------------------------------------------------------------*/
/* Get Drive Status                                                      */
/*-----------------------------------------------------------------------*/

DSTATUS disk_status (
	BYTE pdrv		/* Physical drive nmuber to identify the drive */
)
{
	switch (pdrv) {
    case DEV_FLASH:
        return RES_OK;
    
	}
	return STA_NOINIT;
}



/*-----------------------------------------------------------------------*/
/* Inidialize a Drive                                                    */
/*-----------------------------------------------------------------------*/

DSTATUS disk_initialize (
	BYTE pdrv				/* Physical drive nmuber to identify the drive */
)
{
	switch (pdrv) {
    case DEV_FLASH:
        FlashInit();
        RtcInit();
        return RES_OK;
    
	}
	return STA_NOINIT;
}



/*-----------------------------------------------------------------------*/
/* Read Sector(s)                                                        */
/*-----------------------------------------------------------------------*/

DRESULT disk_read (
	BYTE pdrv,		/* Physical drive nmuber to identify the drive */
	BYTE *buff,		/* Data buffer to store read data */
	LBA_t sector,	/* Start sector in LBA */
	UINT count		/* Number of sectors to read */
)
{
	switch (pdrv) {
    case DEV_FLASH:
        if (FlashUnitRead(sector, count, buff) == false)
        {
            return RES_ERROR;
        }
        return RES_OK;
	}

	return RES_PARERR;
}



/*-----------------------------------------------------------------------*/
/* Write Sector(s)                                                       */
/*-----------------------------------------------------------------------*/

#if FF_FS_READONLY == 0

DRESULT disk_write (
	BYTE pdrv,			/* Physical drive nmuber to identify the drive */
	const BYTE *buff,	/* Data to be written */
	LBA_t sector,		/* Start sector in LBA */
	UINT count			/* Number of sectors to write */
)
{
	switch (pdrv) {
    case DEV_FLASH:
        if (FlashUnitWrite(sector, count, (uint8_t *)buff) == false)
        {
            return RES_ERROR;
        }
        return RES_OK;
	}

	return RES_PARERR;
}

#endif


/*-----------------------------------------------------------------------*/
/* Miscellaneous Functions                                               */
/*-----------------------------------------------------------------------*/

DRESULT disk_ioctl (
	BYTE pdrv,		/* Physical drive nmuber (0..) */
	BYTE cmd,		/* Control code */
	void *buff		/* Buffer to send/receive control data */
)
{
	switch (pdrv) {
    case DEV_FLASH:
        switch (cmd)
        {
            case CTRL_SYNC:
                return RES_OK;
            
            case GET_SECTOR_COUNT:  // 0x200
                *(DWORD *)buff = FlashUnitCount();
                return RES_OK;
            
            case GET_SECTOR_SIZE:   // 0x1000
                *(DWORD *)buff = FlashUnitSize();
                return RES_OK;
            
            case GET_BLOCK_SIZE:    // 
                *(DWORD *)buff = 1;
                return RES_OK;
            
            default:
                return RES_PARERR;
        }
	}

	return RES_PARERR;
}

DWORD get_fattime(void)
{
    uint32_t fattime;
    uint8_t dateTime[7];
    
    RtcRead_Hex(dateTime);
    
    fattime = ((2000 + dateTime[0] - 1980) << 25) | 
            (dateTime[1] << 21) | (dateTime[2] << 16) | 
            (dateTime[3] << 11) | (dateTime[4] << 5) | (dateTime[5] >> 1);

    
    return fattime;
}
